<?php
/**
 * MWPS Core Functions
 *
 * General core functions available on both the front-end and admin.
 */

defined( 'ABSPATH' ) || exit;

/**
 * get data from url
 * @param string $url  URL.
 */
function mwps_get_data( $url ) {
    // check/get if it is a xml feed url
    $rss_xml_feed = mlts_get_rss_xml_feed($url);
    //$rss_xml_feed = 0;
    // not xml - could be a json url
    if(!($rss_xml_feed)) {
        // find '/json/' at the end of URL, append if not found
        $subURL = substr($url, -5);
        if(strpos($subURL, 'json') === FALSE) {
            $url .= '/json/';
        }
        $response = wp_remote_get( $url );
        if ( ! is_wp_error( $response ) ) {
            $result = wp_remote_retrieve_body( $response );
        } else $result = json_encode(['feed_type'=> 'json']);

        $type = ['feed_type'=> 'json'];

    } else {
        $result = json_encode($rss_xml_feed);
        $type = ['feed_type'=> 'xml'];
    }

    $array = json_decode($result, true);
    $array = (is_array($array)) ? $array : [];
    return array_merge($array, $type);
}

// Parse the RSS/XML feed
function mlts_get_rss_xml_feed($url) {
    $response = wp_remote_get( $url );
    if ( ! is_wp_error( $response ) ) {
        $result = wp_remote_retrieve_body( $response );
    } else $result = '';

    if(substr($result, 0, 7) == "<report" ||
        substr($result, 0, 5) == "<?xml" ||
        substr($result, 0, 11) == "<facilities") {
        $xmlstr = new MLTSJsonSerializer($result);
    } else {
        // Feed is not valid, continue and display error below.
        $xmlstr = '';
    }
    return $xmlstr;
}

/*
 * @param string $name Name of option or name of post custom field.
 * @param string $value Optional Attachment ID
 * @return string HTML of the Upload Button
 */
function mlts_image_uploader_field( $rating, $name, $value = '') {
    $image = 'icon-rating-'.$rating.' ">';
    $image_size = 'full'; // it would be better to use thumbnail size here (150x150 or so)
    $display = 'none'; // display state ot the "Remove image" button

    if( $image_attributes = wp_get_attachment_image_src( $value, $image_size ) ) {

        // $image_attributes[0] - image URL
        // $image_attributes[1] - image width
        // $image_attributes[2] - image height

        $image = 'icon-rating-'.$rating.' uploaded"><img src="' . esc_url($image_attributes[0]) . '" style="display:block;" />';
        $display = 'inline-block';

    }

    return '
	<div>
		<span class="statusIcon ' . $image . '</span>
		<input type="hidden" name="' . $name . '" id="' . $name . '" value="' . esc_attr( $value ) . '" />
		<a href="#" class="mlts_remove_image_button" style="display:inline-block;display:' . $display . '">Reset Icon</a>
	</div>';
}

// get areas
function mlts_get_areas($obj) {
    $feed_type = (isset($obj['feed_type'])) ? $obj['feed_type'] : '';
    $areas = [];

    if($feed_type === 'json') {
        if(isset($obj['facilities']) && isset($obj['facilities']['areas'])) {
            $areas = $obj['facilities']['areas'];
        } elseif (isset($obj['areas'])) $areas = $obj['areas'];
    } elseif($feed_type === 'xml') {
        if(isset($obj['facilities']) && isset($obj['facilities']['areas']) &&
            isset($obj['facilities']['areas'][0]) && isset($obj['facilities']['areas'][0]['area'])) {
            $areas = $obj['facilities']['areas'][0]['area'];
        } elseif (isset($obj['report']) && isset($obj['report']['facilities']) &&
            isset($obj['report']['facilities'][0]) && isset($obj['report']['facilities'][0]['areas']) &&
            isset($obj['report']['facilities'][0]['areas'][0]) && isset($obj['report']['facilities'][0]['areas'][0]['area'])) {
            $areas = $obj['report']['facilities'][0]['areas'][0]['area'];
        } elseif (isset($obj['areas']) && isset($obj['areas']['area'])) $areas = $obj['areas']['area'];
    }

    return $areas;
}

// get lifts/trails in areas
function mlts_get_lifts_trails($area, $lifts_trails='lifts', $feed_type='json') {
    if($lifts_trails === 'lifts') $sub_lifts_trails = 'lift';
    else $sub_lifts_trails = 'trail';
    $lifts = [];
    if($feed_type === 'xml') {
        $lifts = ( isset($area[$lifts_trails]) && isset($area[$lifts_trails][0]) && isset($area[$lifts_trails][0][$sub_lifts_trails]) ) ? $area[$lifts_trails][0][$sub_lifts_trails] : [];
    } else $lifts = isset($area[$lifts_trails]) ? $area[$lifts_trails] : [];
    /*if(count($lifts) <= 0 && $lifts_trails === 'trails' && $feed_type === 'xml') {
        $lifts = ( isset($area['freestyleTerrain']) && isset($area['freestyleTerrain'][0]) && isset($area['freestyleTerrain'][0]['parks']) && isset($area['freestyleTerrain'][0]['parks'][0])  && isset($area['freestyleTerrain'][0]['parks'][0]['park']) ) ? $area['freestyleTerrain'][0]['parks'][0]['park'] : [];
    }*/

    return $lifts;
}

// array_map recursive
if(!function_exists('mlts_array_map_recursive')) {
function mlts_array_map_recursive($callback, $array) {
    $func = function ($item) use (&$func, &$callback) {
        return is_array($item) ? array_map($func, $item) : call_user_func($callback, $item);
    };

    return array_map($func, $array);
}
}

/**
 * Class JsonSerializer
 */
if(!class_exists('MLTSJsonSerializer')) {
class MLTSJsonSerializer extends SimpleXmlElement implements JsonSerializable {
    const ATTRIBUTE_INDEX = "@attr";
    const CONTENT_NAME = "_text";

    /**
     * SimpleXMLElement JSON serialization
     *
     * @return array
     *
     * @link http://php.net/JsonSerializable.jsonSerialize
     * @see JsonSerializable::jsonSerialize
     * @see https://stackoverflow.com/a/31276221/36175
     */
    function jsonSerialize() {
        $array = [];

        if ($this->count()) {
            // serialize children if there are children
            /**
             * @var string $tag
             * @var JsonSerializer $child
             */
            foreach ($this as $tag => $child) {
                $temp = $child->jsonSerialize();
                $attributes = [];

                foreach ($child->attributes() as $name => $value) {
                    $attributes["$name"] = (string) $value;
                }

                //$array[$tag][] = array_merge($temp, [self::ATTRIBUTE_INDEX => $attributes]);
                $array[$tag][] = array_merge($temp, $attributes);
            }
        } else {
            // serialize attributes and text for a leaf-elements
            $temp = (string) $this;

            // if only contains empty string, it is actually an empty element
            if (trim($temp) !== "") {
                $array[self::CONTENT_NAME] = $temp;
            }
        }

        if ($this->xpath('/*') == array($this)) {
            // the root element needs to be named
            $array = [$this->getName() => $array];
        }

        return $array;
    }
}
}
